/***************************************************************************\
  "k_track.h" - Kevin's Memory Tracker For Catching Memory Leaks
  Revision 5; 16 Jan 2003 - 21 Feb 2004
  Available at: "http://kevin.alteu.com/"
  E-mail:       "kevin@albrecht.net"

  Copyright (C) 2003-2004, Kevin P. Albrecht

  Permission is hereby granted by Kevin P. Albrecht (the "Author"), free of
  charge, to any person obtaining a copy of this software and associated
  documentation files (the "Software"), to deal in the Software without
  restriction, including without limitation the rights to use, copy, modify,
  merge, publish, distribute, sublicense, and/or sell copies of the
  Software, and to permit persons to whom the Software is furnished to do
  so, subject to the following conditions:

  The above copyright notice and this permission notice shall be included
  in all copies or substantial portions of the Software.

  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
  THE AUTHOR BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN
  AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
  CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.

  Except as contained in this notice, the name of the Author shall not be
  used in advertising or otherwise to promote the sale, use or other
  dealings in this Software without prior written authorization from the
  Author.
\***************************************************************************/

#ifndef K_TRACK_H
#define K_TRACK_H

#ifdef GLOBAL_DEBUG

  /////////////////////////
  // DEFINES //////////////
  /////////////////////////

  // External Defines //
  // GLOBAL_DEBUG or GLOBAL_RELEASE

  /////////////////////////
  // INCLUDES /////////////
  /////////////////////////

  // Standard //

  // Local //

  /////////////////////////
  // DECLARATIONS /////////
  /////////////////////////

  typedef struct TrackUnit_tag
  {
    char *file;
    int line;
    void *pointer;
    struct TrackUnit_tag* next;
    struct TrackUnit_tag* prev;
  } TrackUnit;

  TrackUnit *k_track_head;
  unsigned int k_track_units;
  unsigned int k_track_units_total;

  void k_track_init ();
  void k_track_deinit ();
  void *k_track_alloc (unsigned int size, const char *file, int line);
  void k_track_free (void *pointer, const char *file, int line);

  /////////////////////////
  // DEFINITIONS //////////
  /////////////////////////

  void k_track_init ()
  {
    k_track_head = NULL;
    k_track_units = 0;
    k_track_units_total = 0;
  }

  void k_track_deinit ()
  {
    TrackUnit *temp;

    temp = k_track_head;

    printf   ("**K_TRACK:COMPLETED** %d memory leak(s) found, %d total units allocatated\n", k_track_units, k_track_units_total);

    while (k_track_head != NULL)
    {
      k_track_head = temp->next;

      printf ("**K_TRACK:LEAK** %s(%d), ptr<0x%x>, str<%s>\n", temp->file, temp->line, (int)temp->pointer, (char*)(temp->pointer));

      free (temp->file);
      //NOTE: temp->pointer not deleted here because it should be deleted in the program.
      free (temp);
      temp = k_track_head;
    }
  }

  void *k_track_alloc (unsigned int size, const char *file, int line)
  {
    TrackUnit *temp;
    int n;

    temp = (TrackUnit *) malloc (sizeof (TrackUnit));

    n = strlen (file) * sizeof (char);
    temp->file = (char *) malloc (n + 1);
    strcpy (temp->file, file);
    temp->line = line;
    temp->pointer = malloc (size);
    temp->next = k_track_head;
    temp->prev = NULL;
    if (k_track_head) k_track_head->prev = temp;
    k_track_head = temp;

    k_track_units++;
    k_track_units_total++;

    return temp->pointer;
  }

  void k_track_free (void *pointer, const char *file, int line)
  {
    TrackUnit *curr;

    if (pointer == NULL) return;

    curr = k_track_head;

    while (curr != NULL)
    {
      if (curr->pointer == pointer)
      {
        free (curr->pointer);
        free (curr->file);

        if (curr->prev == NULL)
        {
          k_track_head = curr->next;
          if (k_track_head) k_track_head->prev = NULL;
        }
        else
        {
          curr->prev->next = curr->next;
          if (curr->next) curr->next->prev = curr->prev;
        }

        free (curr);
        k_track_units--;        
        return;
      }

      curr = curr->next;
    }

    printf ("**K_TRACK:CANT_FREE**: %s(%d), ptr<0x%x>, str<%s>\n", file, line, (int)pointer, (char*)(pointer));
  }

#endif

#ifdef GLOBAL_DEBUG
  #define KALLOC(size) k_track_alloc((size), __FILE__, __LINE__)
#else
  #define KALLOC(size) malloc((size))
#endif

#ifdef GLOBAL_DEBUG
  #define KFREE(pointer) k_track_free((pointer), __FILE__, __LINE__)
#else
  #define KFREE(pointer) free(pointer)
#endif

#endif
