/***************************************************************************\
  "rstack.h" - Runtime Stack (Phase E)
  Revision 4; 27 Nov 2002 - 21 Feb 2004
  Copyright (C) 2002-2004  Kevin P. Albrecht
\***************************************************************************/

#ifndef RSTACK_H
#define RSTACK_H

/////////////////////////
// DEFINES //////////////
/////////////////////////

/////////////////////////
// INCLUDES /////////////
/////////////////////////

// Standard //
#include <stdio.h>

// Local //
#include "global.h"
#include "error.h"
#include "table.h"
#include "k_conv.h"

/////////////////////////
// DECLARATIONS /////////
/////////////////////////

// External //
typedef struct
{
  Symbol *top;
  int count;
} RStack;

RStack *rstack_new ();
void rstack_del (RStack *rs);
void rstack_push (RStack *rs, Symbol *sym);
void *rstack_pop (RStack *rs);
void rstack_copytop (RStack *rs);
void rstack_swap (RStack *rs);
Type rstack_peektype1 (RStack *rs);
Type rstack_peektype2 (RStack *rs);

/////////////////////////
// DEFINITIONS //////////
/////////////////////////

RStack *rstack_new ()
{
  RStack *rs = (RStack*) KALLOC(sizeof (RStack));
  rs->top = NULL;
  rs->count = 0;
  return rs;
}

void rstack_del (RStack *rs)
{
  Symbol *temp;

  while (rs->top != NULL)
  {
    temp = rs->top;
    rs->top = rs->top->next;
    sym_del (temp);
  }
  
  KFREE(rs);
}

void rstack_push (RStack *rs, Symbol *sym)
{
  Symbol *temp;

  if (rs->top == NULL)
  {
    rs->top = sym_copy (sym);
    rs->top->next = NULL;
  }
  else
  {
    temp = rs->top;
    rs->top = sym_copy (sym);
    rs->top->next = temp;
  }

  rs->count++;
}

void *rstack_pop (RStack *rs)
{
  Symbol *symtemp;
  void *vtemp;

  if (rs->top != NULL)
  {
    symtemp = rs->top;
    rs->top = rs->top->next;
    rs->count--;
    vtemp = symtemp->value;
    sym_del_savevalue (symtemp);
    return vtemp;
  }
  else
  {
    err (true, ER_STACK_EMPTY, NULL);
    erre (EFF_TERMINATE);
    return NULL;
  }
}

void rstack_copytop (RStack *rs)
{
  Symbol *symbol;

  if (rs->top != NULL)
  {
    symbol = sym_copy (rs->top);
    symbol->next = rs->top;
    rs->top = symbol;
  }
  else
  {
    err (true, ER_STACK_EMPTY, NULL);
    erre (EFF_TERMINATE);
  }
}

void rstack_swap (RStack *rs)
{
  Symbol *a;
  Symbol *b;
  Symbol *c;

  if ((rs->top == NULL) || (rs->top->next == NULL))
  {
    err (true, ER_STACK_EMPTY, NULL);
    erre (EFF_TERMINATE);
  }

  a = rs->top;
  b = rs->top->next;
  c = rs->top->next->next;

  rs->top = b;
  b->next = a;
  a->next = c;
}

Type rstack_peektype1 (RStack *rs)
{
  if (rs->top != NULL)
  {
    return rs->top->type;
  }
  else
  {
    return tNull;
  }
}

Type rstack_peektype2 (RStack *rs)
{
  if ((rs->top != NULL) && (rs->top->next != NULL))
  {
    return rs->top->next->type;
  }
  else
  {
    return tNull;
  }
}

#endif

